library ESP32;

{$R *.res}

uses
  WinProcs,
  WinTypes,
  SysUtils,
  Controls,
  Forms,
  Math,
  Dialogs,
  ESP32Dlg in 'ESP32Dlg.pas' {ESP32Dialog},
  COMUtils in 'COMUtils.pas';

function  Connect(ComPort, Baudrate: integer): boolean; stdcall; external 'WFComPortManager.dll';
procedure Disconnect(ComPort: integer); stdcall; external 'WFComPortManager.dll';
function  GetComHandle(ComPort: integer): THandle; stdcall; external 'WFComPortManager.dll';

type
  PUserData = ^TUserData;
  TUserData = record
    ComPort, Baudrate: integer;
    LastInputs: array[1..50] of extended;
    DAC1Active, DAC2Active, DOut1Active, DOut2Active, ADC1CH4Active, ADC1CH5Active, DIn1Active, DIn2Active: boolean;
  end;

  PParameterStruct=^TParameterStruct;
  TParameterStruct=packed record
    NuE : Byte;                           {Anzahl reeller Zahlenwerte}
    NuI : Byte;                           {Anzahl ganzer Zahlenwerte}
    NuB : Byte;                           {Anzahl Schalter}
    E:Array[0..31] of Extended;           {reelle Zahlenwerte}
    I:Array[0..31] of Integer;            {ganze Zahlenwerte}
    B:Array[0..31] of Byte;               {Schalter}
    D:Array[0..255] of AnsiChar;          {event. Dateiname fr weitere Daten.}
    EMin:Array[0..31] of Extended;        {untere Eingabegrenze fr jeden reellen Zahlenwert}
    EMax:Array[0..31] of Extended;        {obere Eingabegrenze fr jeden rellen Zahlenwert}
    IMin:Array[0..31] of Integer;         {untere Eingabegrenze fr jeden ganzzahligen Zahlenwert}
    IMax:Array[0..31] of Integer;         {obere Eingabegrenze fr jeden ganzzahligen Zahlenwert}
    NaE : Array[0..31,0..40] of AnsiChar; {Namen der reellen Zahlenwerten}
    NaI : Array[0..31,0..40] of AnsiChar; {Namen der ganzen Zahlenwerten}
    NaB : Array[0..31,0..40] of AnsiChar; {Namen der Schalter}
    UserDataPtr: PUserData;               {Zeiger auf weitere Blockvariablen}
    ParentPtr: Pointer;                   {Zeiger auf User-DLL-Block}
    ParentHWnd: HWnd;                     {Handle des User-DLL-Blocks}
    ParentName: PAnsiChar;                {Name des User-DLL-Blocks}
    UserHWindow: HWnd;                    {Benutzerdef. Fensterhandle, z. B. fr Ausgabefenster}
    DataFile: text;                       {Textdatei fr universelle Zwecke}
  end;

  PDialogEnableStruct=^TDialogEnableStruct;
  TDialogEnablestruct=packed record
    AllowE: Longint;                      { Soll die Eingabe eines Wertes   }
    AllowI: Longint;                      { un-/zulssig sein so ist das Bit}
    AllowB: Longint;                      { des Allow?-Feldes 0 bzw. 1}
    AllowD: Byte;
  end;

  PNumberOfInputsOutputs=^TNumberOfInputsOutputs;
  TNumberOfInputsOutputs=packed record
    Inputs :Byte;                         {Anzahl Eingnge}
    Outputs:Byte;                         {Anzahl Ausgnge}
    NameI : Array[0..49,0..40] of AnsiChar;
    NameO : Array[0..49,0..40] of AnsiChar;
  end;

  PInputArray = ^TInputArray;
  TInputArray = packed array[1..30] of extended;
  POutputArray = ^TOutputArray;
  TOutputArray = packed array[1..30] of extended;


procedure GetParameterStruct(D:PParameterStruct);export stdcall;
begin
  //Anzahl der Parameter
  D^.NuE := 0;
  D^.NuI := 0;
  D^.NuB := 0;
end;

procedure GetDialogEnableStruct(D:PDialogEnableStruct;D2:PParameterStruct);export stdcall;
begin
end;

procedure GetNumberOfInputsOutputs(D:PNumberOfInputsOutputs);export stdcall;
begin
  D^.Inputs := 4;
  D^.Outputs := 4;
  D^.NameI[0] := 'DAC1 (GPIO25)';
  D^.NameI[1] := 'DAC2 (GPIO26)';
  D^.NameI[2] := 'DOut1 (GPIO4)';
  D^.NameI[3] := 'DOut2 (GPIO15)';
  D^.NameO[0] := 'ADC1 CH4 (GPIO32)';
  D^.NameO[1] := 'ADC1 CH5 (GPIO33)';
  D^.NameO[2] := 'DIn1 (GPIO34)';
  D^.NameO[3] := 'DIn2 (GPIO35)';
end;

Procedure CallParameterDialogDLL(D1: PParameterStruct; D2: PNumberOfInputsOutputs); export stdcall;
var i: integer;
    DialogForm: TESP32Dialog;
    Dummy: extended;
    s: array[0..1000] of AnsiChar;
begin
  DialogForm := TESP32Dialog.Create(Application);
  with  DialogForm do begin
    UpDown1.Position := D1^.UserDataPtr.ComPort;
    IntegerEdit1.Value := D1^.UserDataPtr.Baudrate;
    Checkbox1.Checked := D1^.UserDataPtr.DAC1Active;
    Checkbox2.Checked := D1^.UserDataPtr.DAC2Active;
    Checkbox3.Checked := D1^.UserDataPtr.DOut1Active;
    Checkbox4.Checked := D1^.UserDataPtr.DOut2Active;
    Checkbox5.Checked := D1^.UserDataPtr.ADC1CH4Active;
    Checkbox6.Checked := D1^.UserDataPtr.ADC1CH5Active;
    Checkbox7.Checked := D1^.UserDataPtr.DIn1Active;
    Checkbox8.Checked := D1^.UserDataPtr.DIn2Active;
    if ShowModal = mrOK then begin
      D1^.UserDataPtr.ComPort := UpDown1.Position;
      D1^.UserDataPtr.Baudrate := IntegerEdit1.Value;
      D1^.UserDataPtr.DAC1Active := Checkbox1.Checked;
      D1^.UserDataPtr.DAC2Active := Checkbox2.Checked;
      D1^.UserDataPtr.DOut1Active := Checkbox3.Checked;
      D1^.UserDataPtr.DOut2Active := Checkbox4.Checked;
      D1^.UserDataPtr.ADC1CH4Active := Checkbox5.Checked;
      D1^.UserDataPtr.ADC1CH5Active := Checkbox6.Checked;
      D1^.UserDataPtr.DIn1Active := Checkbox7.Checked;
      D1^.UserDataPtr.DIn2Active := Checkbox8.Checked;
    end;
    Free;
  end;
end;

function CanSimulateDLL(D:PParameterStruct):Integer; export stdcall;
begin
  Screen.Cursor := crHourGlass;
  if Connect(D^.UserDataPtr.COMPort, D^.UserDataPtr.Baudrate) then with D^.UserDataPtr^ do begin
    Disconnect(D^.UserDataPtr.COMPort);
    Result := 1;
  end else
    Result := 0;
  Screen.Cursor := crDefault;
end;

procedure SimulateDLL(T:Extended;D1:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
var i: integer;
    s: string;
begin
  // Analog Outputs
  if D1.UserDataPtr.DAC1Active then begin
    i := round(Max(Min(Inputs^[1], 255), 0));
    SendString(GetComHandle(D1.UserDataPtr^.ComPort), '!A1' + Format('%.3d', [i]));
    ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
  end;
  if D1.UserDataPtr.DAC2Active then begin
    i := round(Max(Min(Inputs^[2], 255), 0));
    SendString(GetComHandle(D1.UserDataPtr^.ComPort), '!A2' + Format('%.3d', [i]));
    ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
  end;
  // Digital Outputs
  if D1.UserDataPtr.DOut1Active then begin
    if Inputs^[3] > 2.5 then
      SendString(GetComHandle(D1.UserDataPtr^.ComPort), '!D11')
    else
      SendString(GetComHandle(D1.UserDataPtr^.ComPort), '!D10');
    ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
  end;
  if D1.UserDataPtr.DOut2Active then begin
    if Inputs^[4] > 2.5 then
      SendString(GetComHandle(D1.UserDataPtr^.ComPort), '!D21')
    else
      SendString(GetComHandle(D1.UserDataPtr^.ComPort), '!D20');
    ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
  end;
  // Analog Inputs
  if D1.UserDataPtr.ADC1CH4Active then begin
    SendString(GetComHandle(D1.UserDataPtr^.ComPort), '?A1');
    s := ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
    Delete(s, Length(s)-1, 2);
    Outputs^[1] := StrToInt(s);
  end;
  if D1.UserDataPtr.ADC1CH5Active then begin
    SendString(GetComHandle(D1.UserDataPtr^.ComPort), '?A2');
    s := ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
    Delete(s, Length(s)-1, 2);
    Outputs^[2] := StrToInt(s);
  end;
  // Digital Inputs
  if D1.UserDataPtr.DIn1Active then begin
    SendString(GetComHandle(D1.UserDataPtr^.ComPort), '?D1');
    s := ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
    Delete(s, Length(s)-1, 2);
    Outputs^[3] := 5*StrToInt(s);
  end;
  if D1.UserDataPtr.DIn2Active then begin
    SendString(GetComHandle(D1.UserDataPtr^.ComPort), '?D2');
    s := ReadString(GetComHandle(D1.UserDataPtr^.ComPort));
    Delete(s, Length(s)-1, 2);
    Outputs^[4] := 5*StrToInt(s);
  end;
end;

procedure InitSimulationDLL(D:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
var i: integer;
begin
  Connect(D^.UserDataPtr.ComPort, D^.UserDataPtr.Baudrate);
  for i:=1 to 50 do
    D^.UserDataPtr.LastInputs[i] := 999999;
  SimulateDLL(0, D, Inputs, Outputs);
end;

procedure EndSimulationDLL2(D: PParameterStruct);export stdcall;
begin
  Disconnect(D^.UserDataPtr.ComPort);
end;

procedure InitUserDLL(D: PParameterStruct); export stdcall;
begin
  Application.Handle := D^.ParentHWnd;
  D^.UserDataPtr := new(PUserData);
  D^.UserDataPtr.Baudrate := 9600;
  D^.UserDataPtr.DAC1Active := true;
  D^.UserDataPtr.DAC2Active := true;
  D^.UserDataPtr.DOut1Active := true;
  D^.UserDataPtr.DOut2Active := true;
  D^.UserDataPtr.ADC1CH4Active := true;
  D^.UserDataPtr.ADC1CH5Active := true;
  D^.UserDataPtr.DIn1Active := true;
  D^.UserDataPtr.DIn2Active := true;
end;

procedure DisposeUserDLL(D: PParameterStruct); export stdcall;
begin
  Application.Handle := 0;
  dispose(D^.UserDataPtr);
end;

function GetDLLName: PAnsiChar; export stdcall;
begin
  GetDLLName := 'ESP32 IN/Out';
end;

Procedure WriteToFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of AnsiChar;
begin
  with D^.UserDataPtr^ do begin
    StrPCopy(s, IntToStr(ComPort) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(Baudrate) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(DAC1Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(DAC2Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(DOut1Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(DOut2Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(ADC1CH4Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(ADC1CH5Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(DIn1Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(ord(DIn2Active)) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
  end;
end;

Procedure ReadFromFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i, Code: integer;
    s: array[0..1000] of AnsiChar;
procedure ReadOneLine(FHandle:Word; Aps :PAnsiChar);
var i  : Integer;
begin
  i:=0;
  _lRead(FHandle,@Aps[i],1);
  repeat
    inc(i);
    _lRead(FHandle,@Aps[i],1);
  until (Aps[i-1]=#13) and (Aps[i]=#10);
  Aps[i-1]:=#0;
end;
begin
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.ComPort := StrToInt(StrPas(s));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.Baudrate := StrToInt(StrPas(s));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.DAC1Active := boolean(StrToInt(StrPas(s)));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.DAC2Active := boolean(StrToInt(StrPas(s)));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.DOut1Active := boolean(StrToInt(StrPas(s)));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.DOut2Active := boolean(StrToInt(StrPas(s)));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.ADC1CH4Active := boolean(StrToInt(StrPas(s)));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.ADC1CH5Active := boolean(StrToInt(StrPas(s)));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.DIn1Active := boolean(StrToInt(StrPas(s)));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.DIn2Active := boolean(StrToInt(StrPas(s)));
end;

function NumberOfLinesInSystemFile: integer; export stdcall;
begin
end;

procedure IsUserDLL32; export stdcall;
begin
//
end;

procedure IsDemoDLL; export stdcall;
begin
//
end;

{Exportieren der notwendigen Funktionen und Prozeduren }
exports
  WriteToFile,
  ReadFromFile,
  NumberOfLinesInSystemFile,
  GetParameterStruct,
  GetDialogEnableStruct,
  GetNumberOfInputsOutputs,
  CanSimulateDLL,
  InitSimulationDLL,
  SimulateDLL,
  InitUserDLL,
  DisposeUserDLL,
  EndSimulationDLL2,
  GetDLLName,
  CallParameterDialogDLL,
  IsUserDLL32,
  IsDemoDLL;


begin
  {Weitere Initialisierung der DLL}
end.
